<?php
/**
 * Plugin Name: Gateline for PremiumPress
 * Plugin URI: https://gateline.ru
 * Description: Gateline payment gateway for PremiumPress themes
 * Version: 1.0.0
 * Author: Gateline
 * Author URI: https://gateline.ru
 * License:
 * Updated: Jul 2022
 */

class GatelinePaymentGateway
{
    public function add($gateways)
    {
        $yesno_list = array('yes' => 'Enable', 'no' => 'Disable');

        $gw['name']     = 'Gateline';
        $gw['logo']     = plugins_url('images/logo.jpg', __FILE__);
        $gw['function'] = $this->fname('form');
        $gw['website']  = 'https://gateline.ru';
        $gw['callback'] = 'yes';
        $gw['ownform']  = 'yes';
        $gw['fields']   = array(
            1 => array(
                'name' => 'Enable gateway',
                'type' => 'listbox',
                'fieldname' => $gw['function'],
                'list' => $yesno_list,
            ),
            2 => array(
                'name' => 'Site UID',
                'type' => 'text',
                'fieldname' => $this->fname('uid'),
            ),
            3 => array(
                'name' => 'Site password',
                'type' => 'text',
                'fieldname' => $this->fname('passwd'),
            ),
            4 => array(
                'name' => 'Enable test mode (sandbox)',
                'type' => 'listbox',
                'fieldname' => $this->fname('sandbox'),
                'list' => $yesno_list,
                'default' => 'no',
            ),
            5 => array(
                'name' => 'Display name',
                'type' => 'text',
                'fieldname' => $this->fname('displayname'),
                'default' => 'Pay Now',
            ),
        );
        $gw['notes'] = 'Redirect URL: '._ppt(array('links', 'callback'));

        $id = count($gateways) + 1;
        $gateways[$id] = $gw;

        return $gateways;
    }

    public function get_form()
    {
        global $CORE, $userdata;

        $query = array(
            'amount' => $GLOBALS['total'],
            'description' => $GLOBALS['description'],
            'merchant_order_id' => $GLOBALS['orderid'].'-'.$GLOBALS['total'],
            'site' => $this->option('uid'),
            'email' => $userdata->user_email,
        );
        $query['checksum'] = $this->calc_checksum($query);

        $url = 'https://simpleapi.gateline.ru/pay';
        if($this->option('sandbox') == 'yes') {
            $url = 'https://simpleapi.sandbox.gateline.net:18610/pay';
        }

        ob_start();
?>

<form method="POST" enctype="multipart/form-data" action="<?php echo $url ?>">
<?php foreach ($query as $key => $value) { ?>
    <input type="hidden" name="<?php echo $key ?>" value="<?php echo $value ?>">
<?php } ?>
    <input type="submit" value="<?php echo $this->option('displayname') ?>">
</form>

<?php
        $content = ob_get_contents();
        ob_end_clean();

        return $content;
    }

    public function process_callback()
    {
        global $userdata;

        $request = $_GET;

        if (!$this->is_checksum_valid($request)) {
            return 'error';
        }

        if ($request['status'] != 'success') {
            return 'error';
        }

        if (!preg_match('/^(.+)-([^-]+)$/', $request['merchant_order_id'], $matches)) {
            return 'error';
        }

        $order_id = $matches[1];
        $amount = $matches[2];

        $result = core_generic_gateway_callback($order_id, array(
            'gateway_name' => 'Gateline',
            'amount'       => $amount,
            'tax'          => 0,
            'shipping'     => 0, 
            'email'        => $userdata->user_email,
            'description'  => $request['message'],
        ));

        return $result;
    }

    private function fname($name)
    {
        return 'gl_gateway_gateline_'.$name;
    }

    private function option($name)
    {
        return get_option($this->fname($name));
    }

    private function calc_checksum($query)
    {
        unset($query['checksum']);
        ksort($query);

        $kv = [];
        foreach ($query as $key => $value) {
            $kv[] = $key . '=' . $value;
        }
        $content = implode(';', $kv);

        return hash_hmac('sha1', $content, trim($this->option('passwd')));
    }

    private function is_checksum_valid($request)
    {
        $sum = $request['checksum'];
        return ($this->calc_checksum($request) == $sum ? true: false);
    }
}

//1. HOOK INTO THE GATEWAY ARRAY

function gl_gateway_gateline_admin($gateways)
{
    $gw = new GatelinePaymentGateway;
    return $gw->add($gateways);
}

add_action('hook_payments_gateways', 'gl_gateway_gateline_admin');

//2. BUILD THE PAYMENT FORM DATA

function gl_gateway_gateline_form($data = '')
{
    $gw = new GatelinePaymentGateway;
    return $gw->get_form();
}

// 3. HANDLE THE CALLBACK 

function gl_gateway_gateline_callback($order_id)
{
    $gw = new GatelinePaymentGateway;
    return $gw->process_callback();
}

add_action('hook_callback', 'gl_gateway_gateline_callback');

?>
